import React from 'react';
import { Routes, Route } from 'react-router-dom';
import { ThemeProvider, createTheme } from '@mui/material/styles';
import CssBaseline from '@mui/material/CssBaseline';

import Layout from './components/Layout';
import HomePage from './pages/HomePage';
import TestingPage from './pages/TestingPage';
import ResultsPage from './pages/ResultsPage';
import ConfigurationPage from './pages/ConfigurationPage';

const theme = createTheme({
  palette: {
    primary: {
      main: '#1976d2',
    },
    secondary: {
      main: '#dc004e',
    },
    background: {
      default: '#f5f5f5',
    },
  },
  typography: {
    fontFamily: 'Roboto, Arial, sans-serif',
  },
});

function App() {
  return (
    <ThemeProvider theme={theme}>
      <CssBaseline />
      <Layout>
        <Routes>
          <Route path="/" element={<HomePage />} />
          <Route path="/testing" element={<TestingPage />} />
          <Route path="/results" element={<ResultsPage />} />
          <Route path="/configuration" element={<ConfigurationPage />} />
        </Routes>
      </Layout>
    </ThemeProvider>
  );
}

export default App; 